using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Text;
using System.Security.Principal;

namespace DalDemo.DAL
{

    public partial class EmployeesDbGateway : IDisposable
    {

        #region Member Variables

        private NorthwindDbDataContext currentCtx;
        private bool isExternalContext = false;

        #endregion

        #region Constructors

        public EmployeesDbGateway(string connectionString)
        {
            currentCtx = new NorthwindDbDataContext(connectionString);
            SetDataLoadOptions();
            isExternalContext = false;
        }

        internal EmployeesDbGateway(NorthwindDbDataContext dbContext)
        {
            currentCtx = dbContext;
            
            if (null == currentCtx.LoadOptions)
				SetDataLoadOptions();
				
            isExternalContext = true;
        }

        #endregion

        #region Public Methods

        public IList<DalDemo.Entities.Employee> GetEmployees()
        {
            return (from o in currentCtx.Employees
                    select TranslateToEntity(o)
                    ).ToList();
        }

        public DalDemo.Entities.Employee GetEmployee(int EmployeeID)
        {

            return (from o in currentCtx.Employees
                    where o.EmployeeID.Equals(EmployeeID)
                    select TranslateToEntity(o)
                    ).SingleOrDefault();

        }

        public void SaveEmployee(DalDemo.Entities.Employee EmployeeTobeSaved)
        {
            var item = currentCtx.Employees.SingleOrDefault(o => o.EmployeeID == EmployeeTobeSaved.EmployeeID);

            if (item == null)
            {
                DalDemo.DAL.Employee newEmployee = new DalDemo.DAL.Employee(); // Add Employee

				newEmployee.EmployeeID = EmployeeTobeSaved.EmployeeID;
				newEmployee.LastName = EmployeeTobeSaved.LastName;
				newEmployee.FirstName = EmployeeTobeSaved.FirstName;
				newEmployee.Title = EmployeeTobeSaved.Title;
				newEmployee.TitleOfCourtesy = EmployeeTobeSaved.TitleOfCourtesy;
				newEmployee.BirthDate = EmployeeTobeSaved.BirthDate;
				newEmployee.HireDate = EmployeeTobeSaved.HireDate;
				newEmployee.Address = EmployeeTobeSaved.Address;
				newEmployee.City = EmployeeTobeSaved.City;
				newEmployee.Region = EmployeeTobeSaved.Region;
				newEmployee.PostalCode = EmployeeTobeSaved.PostalCode;
				newEmployee.Country = EmployeeTobeSaved.Country;
				newEmployee.HomePhone = EmployeeTobeSaved.HomePhone;
				newEmployee.Extension = EmployeeTobeSaved.Extension;
				newEmployee.Notes = EmployeeTobeSaved.Notes;
				newEmployee.ReportsTo = EmployeeTobeSaved.ReportsTo;
				newEmployee.PhotoPath = EmployeeTobeSaved.PhotoPath;

                currentCtx.Employees.InsertOnSubmit(newEmployee);

            }
            else  // Update Employee
            {

                OnBeforeApplyUpdates(ref EmployeeTobeSaved, ref item);

				item.EmployeeID = EmployeeTobeSaved.EmployeeID;
				item.LastName = EmployeeTobeSaved.LastName;
				item.FirstName = EmployeeTobeSaved.FirstName;
				item.Title = EmployeeTobeSaved.Title;
				item.TitleOfCourtesy = EmployeeTobeSaved.TitleOfCourtesy;
				item.BirthDate = EmployeeTobeSaved.BirthDate;
				item.HireDate = EmployeeTobeSaved.HireDate;
				item.Address = EmployeeTobeSaved.Address;
				item.City = EmployeeTobeSaved.City;
				item.Region = EmployeeTobeSaved.Region;
				item.PostalCode = EmployeeTobeSaved.PostalCode;
				item.Country = EmployeeTobeSaved.Country;
				item.HomePhone = EmployeeTobeSaved.HomePhone;
				item.Extension = EmployeeTobeSaved.Extension;
				item.Notes = EmployeeTobeSaved.Notes;
				item.ReportsTo = EmployeeTobeSaved.ReportsTo;
				item.PhotoPath = EmployeeTobeSaved.PhotoPath;

                OnAfterApplyUpdates(ref EmployeeTobeSaved, ref item);
                
            }

            try
            {
                currentCtx.SubmitChanges(ConflictMode.ContinueOnConflict);
            }
            catch (ChangeConflictException)
            {
                currentCtx.ChangeConflicts.ResolveAll(RefreshMode.KeepCurrentValues); //Employee values will be keeped on the resolution
            }
        }

        public void DeleteEmployee(int EmployeeID)
        {
            var item = currentCtx.Employees.SingleOrDefault(o => o.EmployeeID == EmployeeID);

            if (null != item)
            {
                currentCtx.Employees.DeleteOnSubmit(item);
                currentCtx.SubmitChanges();
            }
        }

        #endregion

        #region Private Methods

        internal  DalDemo.Entities.Employee TranslateToEntity(DalDemo.DAL.Employee dbEmployee)
        {
			if (null == dbEmployee)
				return null;
			
            DalDemo.Entities.Employee entityEmployee = new DalDemo.Entities.Employee();

			entityEmployee.EmployeeID = dbEmployee.EmployeeID;
			entityEmployee.LastName = dbEmployee.LastName;
			entityEmployee.FirstName = dbEmployee.FirstName;
			entityEmployee.Title = dbEmployee.Title;
			entityEmployee.TitleOfCourtesy = dbEmployee.TitleOfCourtesy;
			entityEmployee.BirthDate = dbEmployee.BirthDate;
			entityEmployee.HireDate = dbEmployee.HireDate;
			entityEmployee.Address = dbEmployee.Address;
			entityEmployee.City = dbEmployee.City;
			entityEmployee.Region = dbEmployee.Region;
			entityEmployee.PostalCode = dbEmployee.PostalCode;
			entityEmployee.Country = dbEmployee.Country;
			entityEmployee.HomePhone = dbEmployee.HomePhone;
			entityEmployee.Extension = dbEmployee.Extension;
			entityEmployee.Notes = dbEmployee.Notes;
			entityEmployee.ReportsTo = dbEmployee.ReportsTo;
			entityEmployee.PhotoPath = dbEmployee.PhotoPath;

			TranslateToEntityExtension( dbEmployee, ref entityEmployee);
            return entityEmployee;
        }

        #endregion

		#region Partial Methods

        partial void SetDataLoadOptions();

		partial void TranslateToEntityExtension(DalDemo.DAL.Employee dbEmployee, ref DalDemo.Entities.Employee entityEmployee);

        partial void OnBeforeApplyUpdates(ref DalDemo.Entities.Employee  source, ref DalDemo.DAL.Employee  target);

        partial void OnAfterApplyUpdates(ref DalDemo.Entities.Employee  source, ref DalDemo.DAL.Employee  target);

		#endregion 
		
        #region IDisposable Members

        public void Dispose()
        {
			// Only displose of the context if this class created it.
            if (!isExternalContext && currentCtx != null)
                currentCtx.Dispose();
        }

        #endregion
	
	}
	
}


