﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Linq;

namespace DalDemo.DAL
{
    
    public partial class OrdersDbGateway
    {
    
        /// <summary>
        /// Provides the opportunity to tell LINQ to SQL which classes 
        /// to "Eager Load" or "Prefetch" when selecting records from 
        /// the database.
        /// </summary>
        partial void SetDataLoadOptions()
        {
            DataLoadOptions op = new DataLoadOptions();
            op.LoadWith<Order>(o => o.Employee);
            currentCtx.LoadOptions = op;
        }

        /// <summary>
        /// Provides an opportunity to modify the entity 
        /// while it is being mapped from the DB to the DTO.
        /// </summary>
        /// <param name="dbOrder">The DB entity.</param>
        /// <param name="entityOrder">The DTO entity.</param>
        partial void TranslateToEntityExtension(Order dbOrder, ref DalDemo.Entities.Order entityOrder)
        {
            entityOrder.Metadata = new Entities.OrderMetadata();
            entityOrder.Metadata.EmployeeName = dbOrder.Employee.FirstName + " " + dbOrder.Employee.LastName;
            // entityOrder.Metadata.Title = dbOrder.Employee.Title;
        }


        /// <summary>
        /// Get all orders that a given employee has processed.
        /// </summary>
        /// <param name="EmployeeID">EmployeeID of the employee.</param>
        /// <returns>A list of orders that this employee processed.</returns>
        public IList<Entities.Order> GetOrdersByEmployeeId(int EmployeeID)
        {
            return (from o in currentCtx.Orders
                    where o.Employee.EmployeeID == EmployeeID
                    select TranslateToEntity(o)
                    ).ToList();
        }
    
    }

}
