using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Text;
using System.Security.Principal;

namespace DalDemo.DAL
{

    public partial class OrdersDbGateway : IDisposable
    {

        #region Member Variables

        private NorthwindDbDataContext currentCtx;
        private bool isExternalContext = false;

        #endregion

        #region Constructors

        public OrdersDbGateway(string connectionString)
        {
            currentCtx = new NorthwindDbDataContext(connectionString);
            SetDataLoadOptions();
            isExternalContext = false;
        }

        internal OrdersDbGateway(NorthwindDbDataContext dbContext)
        {
            currentCtx = dbContext;
            
            if (null == currentCtx.LoadOptions)
				SetDataLoadOptions();
				
            isExternalContext = true;
        }

        #endregion

        #region Public Methods

        public IList<DalDemo.Entities.Order> GetOrders()
        {
            return (from o in currentCtx.Orders
                    select TranslateToEntity(o)
                    ).ToList();
        }

        public DalDemo.Entities.Order GetOrder(int OrderID)
        {

            return (from o in currentCtx.Orders
                    where o.OrderID.Equals(OrderID)
                    select TranslateToEntity(o)
                    ).SingleOrDefault();

        }

        public void SaveOrder(DalDemo.Entities.Order OrderTobeSaved)
        {
            var item = currentCtx.Orders.SingleOrDefault(o => o.OrderID == OrderTobeSaved.OrderID);

            if (item == null)
            {
                DalDemo.DAL.Order newOrder = new DalDemo.DAL.Order(); // Add Order

				newOrder.OrderID = OrderTobeSaved.OrderID;
				newOrder.CustomerID = OrderTobeSaved.CustomerID;
				newOrder.EmployeeID = OrderTobeSaved.EmployeeID;
				newOrder.OrderDate = OrderTobeSaved.OrderDate;
				newOrder.RequiredDate = OrderTobeSaved.RequiredDate;
				newOrder.ShippedDate = OrderTobeSaved.ShippedDate;
				newOrder.ShipVia = (int?)OrderTobeSaved.ShippingMethod;
				newOrder.Freight = OrderTobeSaved.Freight;
				newOrder.ShipName = OrderTobeSaved.ShipName;
				newOrder.ShipAddress = OrderTobeSaved.ShipAddress;
				newOrder.ShipCity = OrderTobeSaved.ShipCity;
				newOrder.ShipRegion = OrderTobeSaved.ShipRegion;
				newOrder.ShipPostalCode = OrderTobeSaved.ShipPostalCode;
				newOrder.ShipCountry = OrderTobeSaved.ShipCountry;

                currentCtx.Orders.InsertOnSubmit(newOrder);

            }
            else  // Update Order
            {

                OnBeforeApplyUpdates(ref OrderTobeSaved, ref item);

				item.OrderID = OrderTobeSaved.OrderID;
				item.CustomerID = OrderTobeSaved.CustomerID;
				item.EmployeeID = OrderTobeSaved.EmployeeID;
				item.OrderDate = OrderTobeSaved.OrderDate;
				item.RequiredDate = OrderTobeSaved.RequiredDate;
				item.ShippedDate = OrderTobeSaved.ShippedDate;
				item.ShipVia = (int?)OrderTobeSaved.ShippingMethod;
				item.Freight = OrderTobeSaved.Freight;
				item.ShipName = OrderTobeSaved.ShipName;
				item.ShipAddress = OrderTobeSaved.ShipAddress;
				item.ShipCity = OrderTobeSaved.ShipCity;
				item.ShipRegion = OrderTobeSaved.ShipRegion;
				item.ShipPostalCode = OrderTobeSaved.ShipPostalCode;
				item.ShipCountry = OrderTobeSaved.ShipCountry;

                OnAfterApplyUpdates(ref OrderTobeSaved, ref item);
                
            }

            try
            {
                currentCtx.SubmitChanges(ConflictMode.ContinueOnConflict);
            }
            catch (ChangeConflictException)
            {
                currentCtx.ChangeConflicts.ResolveAll(RefreshMode.KeepCurrentValues); //Order values will be keeped on the resolution
            }
        }

        public void DeleteOrder(int OrderID)
        {
            var item = currentCtx.Orders.SingleOrDefault(o => o.OrderID == OrderID);

            if (null != item)
            {
                currentCtx.Orders.DeleteOnSubmit(item);
                currentCtx.SubmitChanges();
            }
        }

        #endregion

        #region Private Methods

        internal  DalDemo.Entities.Order TranslateToEntity(DalDemo.DAL.Order dbOrder)
        {
			if (null == dbOrder)
				return null;
			
            DalDemo.Entities.Order entityOrder = new DalDemo.Entities.Order();

			entityOrder.OrderID = dbOrder.OrderID;
			entityOrder.CustomerID = dbOrder.CustomerID;
			entityOrder.EmployeeID = dbOrder.EmployeeID;
			entityOrder.OrderDate = dbOrder.OrderDate;
			entityOrder.RequiredDate = dbOrder.RequiredDate;
			entityOrder.ShippedDate = dbOrder.ShippedDate;
			entityOrder.ShippingMethod = (Entities.ShippingMethodEnum?)dbOrder.ShipVia;
			entityOrder.Freight = dbOrder.Freight;
			entityOrder.ShipName = dbOrder.ShipName;
			entityOrder.ShipAddress = dbOrder.ShipAddress;
			entityOrder.ShipCity = dbOrder.ShipCity;
			entityOrder.ShipRegion = dbOrder.ShipRegion;
			entityOrder.ShipPostalCode = dbOrder.ShipPostalCode;
			entityOrder.ShipCountry = dbOrder.ShipCountry;

			TranslateToEntityExtension( dbOrder, ref entityOrder);
            return entityOrder;
        }

        #endregion

		#region Partial Methods

        partial void SetDataLoadOptions();

		partial void TranslateToEntityExtension(DalDemo.DAL.Order dbOrder, ref DalDemo.Entities.Order entityOrder);

        partial void OnBeforeApplyUpdates(ref DalDemo.Entities.Order  source, ref DalDemo.DAL.Order  target);

        partial void OnAfterApplyUpdates(ref DalDemo.Entities.Order  source, ref DalDemo.DAL.Order  target);

		#endregion 
		
        #region IDisposable Members

        public void Dispose()
        {
			// Only displose of the context if this class created it.
            if (!isExternalContext && currentCtx != null)
                currentCtx.Dispose();
        }

        #endregion
	
	}
	
}


