﻿using System.CodeDom;
using System.CodeDom.Compiler;
using System.Reflection;
using System.IO;
using Microsoft.CSharp;

namespace T4ConsoleApp.CodeDOM
{
    /// <summary>
    /// CodeDOM code that generates a Customer class.
    /// </summary>
    public class Customer
    {
        // Member Variables		
        private readonly string _fileName;

        public Customer(string strFileName)
        {
            _fileName = strFileName;
        }

        /// <summary>
        /// Generates a Customer class in C# or VB.NET
        /// </summary>
        /// <param name="path">path to the file to be created</param>
        /// <param name="codeDomProvider">In this example, either CSharpCodeProvider or VBCodeProvider</param>
        public void CreateCodeDomCustomer(string path, CodeDomProvider codeDomProvider)
        {
            // Initialize CodeDom Variables		
            string fileSuffix = (codeDomProvider is CSharpCodeProvider) ? ".cs" : ".vb";
            Stream s = File.Open(path + _fileName + fileSuffix, FileMode.Create);
            StreamWriter sw = new StreamWriter(s);

            // Use the interface (ICodeGenerator) for generating code
            ICodeGenerator codeGenerator = codeDomProvider.CreateGenerator(sw);
            CodeGeneratorOptions cop = new CodeGeneratorOptions();

            // Create Class Namespaces
            CodeNamespace cnsCodeDom = new CodeNamespace("CodeDom");

            // Create Class Declaration
            CodeTypeDeclaration ctd = new CodeTypeDeclaration();
            ctd.IsClass = true;
            ctd.Name = "Customer";
            ctd.TypeAttributes = TypeAttributes.Public;

            // Create Class Member FIELDS	
            sw.WriteLine();
            CodeMemberField cmfCustomerName = new CodeMemberField(typeof(string), "_CustomerName");
            cmfCustomerName.Attributes = MemberAttributes.Private;
            ctd.Members.Add(cmfCustomerName);

            // Create Class CONSTRUCTOR				
            CodeConstructor ccon = new CodeConstructor();
            ccon.Attributes = MemberAttributes.Public;
            ccon.Statements.Add(new CodeCommentStatement(""));
            ccon.Statements.Add(new CodeCommentStatement(" TODO: Add constructor logic here "));
            ccon.Statements.Add(new CodeCommentStatement(""));
            ctd.Members.Add(ccon);

            // Create Class CustomerName PROPERTY
            CodeMemberProperty mpCustomerName = new CodeMemberProperty();
            mpCustomerName.Attributes = MemberAttributes.Public;
            mpCustomerName.Type = new CodeTypeReference(typeof(string));
            mpCustomerName.Name = "CustomerName";
            mpCustomerName.GetStatements.Add(new CodeMethodReturnStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "_CustomerName")));
            mpCustomerName.SetStatements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "_CustomerName"), new CodePropertySetValueReferenceExpression()));
            ctd.Members.Add(mpCustomerName);

            // Create Class GetCustomerName METHOD
            CodeMemberMethod mtd1 = new CodeMemberMethod();
            mtd1.Name = "GetCustomerName";
            mtd1.ReturnType = new CodeTypeReference(typeof(string));
            mtd1.Attributes = MemberAttributes.Public;
            mtd1.Statements.Add(new CodeMethodReturnStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "_CustomerName")));
            ctd.Members.Add(mtd1);

            // Generate Source Code File
            cnsCodeDom.Types.Add(ctd);
            codeGenerator.GenerateCodeFromNamespace(cnsCodeDom, sw, cop);

            // Close StreamWriter
            sw.Close();
            s.Close();
        }
    }
}